// The following program includes features such as 
// 1. Dynamic Menu Association 
// 2. Status & Tool Bar Implementation 
// 3. Rubber Band Effect 
// 4. Displaying text on client area 
// 5. Using the standard color dialog box 
// 6. Displaying Mouse Cursor position on the status bar 
// 7. Displaying a Float Menu(Short Cut Menu)

#include <afxwin.h>
#include <afxext.h>
#include <afxcoll.h>
#include <stdio.h>
#include <string.h>
#include "resource.h"

/* --- Status Bar Variables --- */
UINT status[]={0,ID_INDICATOR_SCRL,ID_INDICATOR_CAPS,ID_INDICATOR_NUM,0,0,0};

/* --- Rubber Band Effect Variables --- */
int choice=0;
static CPoint startp,prevp,currp;
BOOL ftime=FALSE,lbdown=FALSE;

/* --- Float Menu Variables --- */
BOOL ShowFloatMenu=FALSE;


/* ----- Class derived from CFrameWnd class ----- */
class MyWindow:public CFrameWnd
{
protected:
	CStatusBar	sb;
	CToolBar	tb;
	CMenu		*MyMenu;
	UINT		m_nTimer;
public:
	MyWindow();			//Constructor
	~MyWindow();
	/* --- Function prototyping for ON_COMMAND macros --- */
	void Flower();		
	void Exit();		
	void FloatMenu();	
	void Face();		
	void Star();		
	void Green();		
	void Red();			
	void Blue();		
	void None();		
	void Line();		
	void Ellips();		
	void Rect();		
	void drawshape(int,int,int,int,CPoint,CPoint);

	/* -- Protyping for System defined functions -- */
	void OnMouseMove(UINT,CPoint);
	void OnRButtonDown(UINT,CPoint);
	void OnLButtonDown(UINT,CPoint);
	void OnLButtonUp(UINT,CPoint);
	void OnChar(UINT nChar,UINT nRep,UINT Flags);
	void OnTimer(UINT nIDEvent);

	DECLARE_MESSAGE_MAP()
};
/* ----- MESSAGE MAP ENTRIES ----- */
BEGIN_MESSAGE_MAP(MyWindow,CFrameWnd)
	ON_COMMAND(ID_CBOX,Flower)
	ON_COMMAND(ID_EXIT,Exit)
	ON_COMMAND(ID_FLOWER,Flower)
	ON_COMMAND(ID_FACE,Face)
	ON_COMMAND(ID_STAR,Star)
	ON_COMMAND(ID_RED,Red)
	ON_COMMAND(ID_BLUE,Blue)
	ON_COMMAND(ID_GREEN,Green)
	ON_COMMAND(ID_FLOAT,FloatMenu)
	ON_COMMAND(ID_NONE,None)
	ON_COMMAND(ID_LINE,Line)
	ON_COMMAND(ID_ELLIPSE,Ellips)
	ON_COMMAND(ID_RECT,Rect)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_WM_CHAR()
	ON_WM_TIMER()
END_MESSAGE_MAP()

MyWindow::MyWindow()
{
	/* --- Dynamic association of menu with CFrameWnd(Window) --- */
	MyMenu=new CMenu;
	Create(0,"NIIT Front Office");
	MyMenu->LoadMenu(MAKEINTRESOURCE(IDR_MENU1));
	SetMenu(MyMenu);
	/* --- Creating Status Bar and entering values --- */
	sb.Create(this);
	sb.SetIndicators(status,(sizeof(status)/sizeof(UINT)));
	sb.SetPaneInfo(0,ID_SEPARATOR,SBPS_STRETCH,300);
	sb.SetPaneInfo(4,ID_EXTRA,SBPS_NOBORDERS,10);
	sb.SetPaneInfo(5,ID_XPOS,SBPS_NORMAL,50);
	sb.SetPaneInfo(6,ID_YPOS,SBPS_NORMAL,50);
	//sb.SetPaneInfo(7,ID_EXTRA,SBPS_POPOUT,70);

	/* --- Creating Tool Bar and attaching it to CFrameWnd(Window) --- */
	tb.Create(this,WS_CHILD|WS_VISIBLE|CBRS_LEFT|CBRS_TOOLTIPS|CBRS_FLYBY|CBRS_GRIPPER);
	tb.LoadToolBar(MAKEINTRESOURCE(IDR_TOOLBAR1));
	tb.EnableDocking(CBRS_ALIGN_ANY);

	/* --- Setting the Timer --- */
	m_nTimer = SetTimer(1, 1000, 0);
}

MyWindow::~MyWindow()
{
	KillTimer(m_nTimer); 
}

void MyWindow::Flower()
{
	CColorDialog colorbox;		// Creating a standard Color Dialog Object
	
	// Checking if the color dialog was closed by clicking "OK" button
	if(colorbox.DoModal()==IDOK)
	{
		COLORREF color;
		color=colorbox.GetColor();
		//Displaying Text on client area based on the color chosen from Color Dialog
		CClientDC dc(this);
		dc.SetTextColor(color);
		dc.TextOut(300,120,"How do you like this color"); 
	}
}

void MyWindow::Face()
{
	CClientDC ConDC(this);
	// Displaying Concentric Circles
	for(int i=100;i<200;i=i+3)
	{
		ConDC.Ellipse(i,i,400-i,400-i);
	}
}

void MyWindow::Star()
{
	CClientDC StarDC(this);
	StarDC.TextOut(300,100,"TAMILIAN UGADI");
}

void MyWindow::Green()
{
	CClientDC TextDC(this);
	TextDC.SetTextColor(RGB(0,255,0));
	TextDC.TextOut(300,140,"This Text is in Green");
}

void MyWindow::Blue()
{
	CClientDC TextDC(this);
	TextDC.SetTextColor(RGB(0,0,255));
	TextDC.TextOut(300,180,"This Text is in Blue");
}

void MyWindow::Red()
{
	CClientDC TextDC(this);
	TextDC.SetTextColor(RGB(255,0,0));
	TextDC.TextOut(300,160,"This Text is in Red");
}

void MyWindow::OnChar(UINT nChar,UINT nRep,UINT Flags)
{
	char str[5];
	sprintf(str,"%c",nChar);
	CClientDC TextDC(this);
	TextDC.TextOut(100,300,"   ");
	TextDC.TextOut(100,300,str);
}

void MyWindow::Exit()
{
	PostQuitMessage(0);
}

void MyWindow::OnMouseMove(UINT u,CPoint cp)
{
	/* ----- Displaying mouse co-ordinates on the status bar ----- */
	int x=cp.x;int y=cp.y;
	char str[10];
	sprintf(str,"Col %d",x);
	sb.SetPaneText(5,str,TRUE);
	sprintf(str,"Row %d",y);
	sb.SetPaneText(6,str,TRUE);
	
	/* ----- RUBBER BAND EFFECT IMPLEMENTATION ----- */
	if(choice==0)
		return;
	
	if(!lbdown)
		return;

	if(ftime)
	{
		currp=cp;
		ftime=FALSE;
		return;
	}

	prevp=currp;
	currp=cp;
	drawshape(choice,255,255,255,startp,prevp);
	drawshape(choice,0,0,0,startp,currp);
}

void MyWindow::OnTimer(UINT nIDEvent) 
{
	SYSTEMTIME	sysTime;
	char		str[80];
	CClientDC	dc(this);

	::GetLocalTime(&sysTime);

	wsprintf(str, "%04d/%02d/%02d - %02d:%02d:%02d:%04d", 
		sysTime.wYear, sysTime.wMonth, sysTime.wDay,
		sysTime.wHour, sysTime.wMinute, sysTime.wSecond, sysTime.wMilliseconds);

   dc.TextOut(200,200,str);


   // Call base class handler.
   CFrameWnd::OnTimer(nIDEvent);
}



void MyWindow::FloatMenu()
{
	ShowFloatMenu=TRUE;	
}

void MyWindow::None()
{
	choice=0;
}

void MyWindow::Line()
{
	choice=1;
	CClientDC ShapeDC(this);
	ShapeDC.MoveTo(40,50);
	ShapeDC.LineTo(40,90);
}

void MyWindow::Ellips()
{
	choice=3;
	CClientDC ShapeDC(this);
	ShapeDC.Ellipse(10,160,90,220);
}

void MyWindow::Rect()
{
	choice=2;
	CClientDC ShapeDC(this);
	ShapeDC.Rectangle(10,275,80,390);
}

/* Displaying Shortcut menu using mouse's right button */
void MyWindow::OnRButtonDown(UINT flags,CPoint cp)
{
	if(!ShowFloatMenu)
		return;

	CMenu *ShortCutPtr;
	ShortCutPtr=MyMenu->GetSubMenu(0);
	ShortCutPtr->TrackPopupMenu(TPM_RIGHTBUTTON,cp.x,cp.y,this,NULL);
}

void MyWindow::drawshape(int shape,int R,int G,int B,CPoint p1,CPoint p2)
{
	CClientDC MyDC(this);
	
	CPen *ColorPen, *OldPen;
	ColorPen=new CPen;
	ColorPen->CreatePen(PS_SOLID,1,RGB(R,G,B));
	OldPen=MyDC.SelectObject(ColorPen);

	switch(shape)
	{
	case 1:
		MyDC.MoveTo(p1.x,p1.y);
		MyDC.LineTo(p2.x,p2.y);
		break;
	case 2:
		MyDC.Rectangle(p1.x,p1.y,p2.x,p2.y);
		break;

	case 3:
		MyDC.Ellipse(p1.x,p1.y,p2.x,p2.y);
		break;
	}
}

void MyWindow::OnLButtonDown(UINT u,CPoint sp)
{
	if (choice==0)
		return;

	ftime=TRUE;
	lbdown=TRUE;
	startp=sp;
}

void MyWindow::OnLButtonUp(UINT u,CPoint lp)
{
	currp=lp;
	drawshape(choice,0,0,0,startp,currp);
	lbdown=FALSE;
}

/* ------------------------------------------------------------- */
/* ----- End of function definitions for ON_COMMAND macros ----- */
/* ------------------------------------------------------------- */

/* ----- Class derived from CWinApp class ----- */
class MyApp:public CWinApp
{
public:
	BOOL InitInstance();
	BOOL ExitInstance();
};

/* First function to be called when an application starts */
BOOL MyApp::InitInstance()
{
	MyWindow *MyWindowObject;
	MyWindowObject=new MyWindow;
	MyWindowObject->UpdateWindow();
	MyWindowObject->ShowWindow(3);
	//sending the address of the window object to the window
	m_pMainWnd=MyWindowObject; 
	return TRUE;
}

/* Last function to be called when an application end */
BOOL MyApp::ExitInstance()
{
	::MessageBox(0,"ExitInstance Called","Exit",0);
	return TRUE;
}

/* ------------ V V V important --------------*/
/* ----- Creation of Application Object ----- */
MyApp MyApplicationObject;
